#include "visualizer.hpp"
#ifndef DISABLE_VISUALIZER

namespace Visualizer {
	uint8_t hid_buf[RAW_EPSIZE]; // Write buffer
	struct mpd_connection *conn;

	FFT_Visualizer fft = FFT_Visualizer(
		width, height, MIN_HZ, MAX_HZ
	);

	std::chrono::time_point<
		std::chrono::steady_clock,
		std::chrono::nanoseconds
	> t;

	std::chrono::time_point<
		std::chrono::steady_clock,
		std::chrono::nanoseconds
	> last_fifo_sync;

	// buffer size for waveform:
	// (44100 / fps * 10), make 10 bigger for slower scrolling
	//
	// Double both buffer sizes if stereo
		Buffer buf = Buffer(
		"/tmp/mpd.fifo",
		44100 / 2, // Keep 500ms of data in buffer
		fft.compute_buffer_output_size()
	);

	void init() {
		// Frame rate limiter
		t = std::chrono::steady_clock::now();
		last_fifo_sync = std::chrono::steady_clock::now();
		conn = mpd_connection_new(NULL, 0, 0);
	}

	void cleanup() {
		mpd_connection_free(Visualizer::conn);
	}

	void fn(Ergodox &Dox) {
		if (std::chrono::steady_clock::now() > t + std::chrono::milliseconds(30)) {
			if (Dox.get_animation_mode() == 0x02) {

				// Animation data type
				hid_buf[1] = CMD_ANIM_DATA_fft;

				if (std::chrono::steady_clock::now() > last_fifo_sync + std::chrono::seconds(10)) {
					mpd_run_disable_output(conn, 1);
					mpd_run_enable_output(conn, 1);
					last_fifo_sync = std::chrono::steady_clock::now();
					spdlog::info("Synchronized fifo");
				}

				buf.update();
				fft.update(buf);

				for (size_t i = 0; i < 10; i++) {
					// Get height from fft, apply bottom_skip
					ssize_t h = fft.get_output()[i] - BOTTOM_SKIP;

					// Enforce max and min
					// max implicitly enforces top_skip
					h = h>KB_RESOLUTION ? KB_RESOLUTION : h;
					h = h<0 ? 0 : h;

					hid_buf[i + 1] = h;
				}
				Dox.write(CMD_ANIM_DATA, hid_buf, Dox.packet_size);
			}


			t = std::chrono::steady_clock::now();
		}
	}
}

#endif